/** @jsx createElement */
import {
  createElement,
  Component,
  findDOMNode,
} from 'rax';
import View from 'nuke-view';
import Image from 'nuke-image';

const PRELOADNUM = 3;
import styles from '../styles/index';

export class CenterImage extends Component {
  constructor() {
    super();
    this.state = {
      error: false,
      loaded: false,
    };
    this.dom = null;
  }

  findDom() {
    if (!this.dom) {
      this.dom = findDOMNode(this.refs.img);
    }
    return this.dom;
  }
  render() {
    const { error } = this.state,
      { index, current, lazysrc, onLoad, ...childProps } = this.props,
      img = (
        <Image
          ref="img"
          onLoad={this.onImgLoad.bind(this)}
          src={lazysrc}
          {...childProps}
        />
      );

    // init first image, others have been preloaded
    if (index === current) {
      return img;
    }

    if (error) {
      return <Error />;
    }

    return img;
  }

  setStyle(styleData) {
    const elementStyle = this.findDom().style;
    for (const styleName in styleData) {
      elementStyle[styleName] = styleData[styleName];
    }
  }
  onImgLoad(e) {

    this.setState({ loaded: true });

    const target = e.target,
      h = target.naturalHeight,
      w = target.naturalWidth,
      r = h / w,
      height = window.innerHeight || window.screen.availHeight,
      width = window.innerWidth || window.screen.availWidth,
      rate = height / width;

    const imgStyle = {};

    if (r >= 3.5) {
      // imgStyle.width = width + "px";
      // imgStyle.height = h * width / w + "px";
      target.setAttribute('long', true);
    }

    if (r > rate) {
      imgStyle.height = `${height}px`;
      imgStyle.width = `${(w * height) / h}px`;
      imgStyle.left = `${width / 2 - (w * height) / h / 2}px`;
    } else if (r < rate) {
      imgStyle.width = `${width}px`;
      imgStyle.height = `${(h * width) / w}px`;
      imgStyle.top = `${height / 2 - (h * width) / w / 2}px`;
    } else {
      imgStyle.width = width;
      imgStyle.height = height;
    }
    this.setStyle(imgStyle);

    target.setAttribute('rate', 1 / r);
    this.props.onLoad && this.props.onLoad(e, this.props.index);
  }
}

const Loading = () => (
  <View className="spinner" style={styles.spinner}>
    <View className="double-bounce1" style={styles.doubleBounce1} />
    <View className="double-bounce2" style={styles.doubleBounce2} />
  </View>
);

const Error = () => (
  <View className="errorpage" style={styles.errorpage}>
    加载失败
  </View>
);
