/** @jsx createElement */
import { createElement, Component, findDOMNode, PropTypes } from 'rax';
import Finger from './finger';
import Transform from './transform.js';
import { CenterImage } from './components.js';
import View from 'nuke-view';
import styles from '../styles/index';

const MARGIN = 30;

class ImageView extends Component {
  static defaultProps = {
    gap: MARGIN,
    current: 0,
    disablePageNum: true,
    desc: '',
    maxScale: 2,
  };

  constructor(props) {
    super();
    this.arrLength = props.imageList.length;
    this.state = {
      current: props.current,
    };
    this.onOrientationChange = this.onOrientationChange.bind(this);
    this.initScale = 1;
    this.screenWidth = window.innerWidth || window.screen.availWidth;
    this.screenHeight = window.innerHeight || window.screen.availHeight;
    this.list = null;
    this.ob = null;
    this.focused = null;
  }

  onOrientationChange() {
    // // 方向改变后新的innerHeight生效需要delay
    // setTimeout(() => {
    //   this.screenWidth = window.innerWidth || window.screen.availWidth;
    //   this.screenHeight = window.innerHeight || window.screen.availHeight;
    //   this.changeIndex(this.state.current);
    // }, 100);
  }

  render() {
    const { desc, disablePageNum, imageList, children, gap, onLoad } = this.props;
    return (
      <View className="imageview" style={styles.imageview}>
        <Finger
          onSingleTap={this.onSingleTap.bind(this)}
          onPressMove={this.onPressMove.bind(this)}
          onSwipe={this.onSwipe.bind(this)}
        >
          <View
            ref={(node) => {
              this.list = findDOMNode(node);
            }}
            className="imagelist"
            style={styles.imagelist}
          >
            {imageList.map((item, i) => (
              <View
                className="imagelist-item"
                style={[styles['imagelist-item'], { marginRight: `${gap}px` }]}
                key={`img${i}`}
              >
                <Finger
                  onPressMove={this.onPicPressMove.bind(this)}
                  onMultipointStart={this.onMultipointStart.bind(this)}
                  onLongTap={this.onLongTap.bind(this)}
                  onPinch={this.onPinch.bind(this)}
                  onRotate={this.onRotate.bind(this)}
                  onMultipointEnd={this.onMultipointEnd.bind(this)}
                  onDoubleTap={this.onDoubleTap.bind(this)}
                >
                  <CenterImage
                    id={`view${i}`}
                    style={[
                      styles.img,
                      {
                        position: 'absolute',
                        pointerEvents: 'none',
                        webkitUserSelect: 'none',
                        userSelect: 'none',
                      },
                    ]}
                    src={item}
                    index={i}
                    current={this.state.current}
                    onLoad={(e) => { }}
                  />
                </Finger>
              </View>
            ))}
          </View>
        </Finger>
        {disablePageNum ? null : (
          <View className="page" ref="page">
            {this.state.current + 1} / {this.arrLength}
          </View>
        )}
        {desc ? <View dangerouslySetInnerHTML={{ __html: desc }} /> : null}
        {children}
      </View>
    );
  }

  componentDidMount() {
    const { current } = this.state,
      { imageList, initCallback } = this.props;

    this.arrLength = imageList.length;

    Transform(this.list);

    current && this.changeIndex(current, false);

    this.bindStyle(current);

    initCallback && initCallback();
    // window.addEventListener('orientationchange', this.onOrientationChange);
  }

  componentWillUnmount() {
    // window.removeEventListener('orientationchange', this.onOrientationChange);
  }

  onSingleTap() {
    this.props.close && this.props.close();
  }

  onPressMove(evt) {
    const { current } = this.state;

    this.endAnimation();

    if (!this.focused) {
      if (
        (current === 0 && evt.deltaX > 0) ||
        (current === this.arrLength - 1 && evt.deltaX < 0)
      ) {
        this.list.translateX += evt.deltaX / 3;
        // console.log('translateX', this.list.translateX);
      } else {
        this.list.translateX += evt.deltaX;
      }
    }

    evt.preventDefault();
  }

  onSwipe(evt) {
    const { direction } = evt;

    let { current } = this.state;
    if (this.focused) {
      return false;
    }
    switch (direction) {
      case 'Left':
        current < this.arrLength - 1 && ++current && this.bindStyle(current);
        break;
      case 'Right':
        current > 0 && current-- && this.bindStyle(current);
        break;
    }
    this.changeIndex(current);
  }

  onPicPressMove(evt) {
    const { deltaX, deltaY } = evt,
      isLongPic = this.ob.getAttribute('long'),
      { scaleX, width } = this.ob;

    if (this.ob.scaleX <= 1 || evt.touches.length > 1) {
      return;
    }

    if (this.ob && this.checkBoundary(deltaX, deltaY)) {
      !isLongPic && (this.ob.translateX += deltaX);
      this.ob.translateY += deltaY;

      if (isLongPic && scaleX * width === this.screenWidth) {
        this.focused = false;
      } else {
        this.focused = true;
      }
    } else {
      this.focused = false;
    }
    // console.log('translate ',this.ob.translateX, this.ob.translateY);
  }

  onMultipointStart() {
    this.initScale = this.ob.scaleX;
  }

  onPinch(evt) {
    if (this.props.disablePinch || this.ob.getAttribute('long')) {
      return false;
    }
    this.ob.style.webkitTransition = 'cubic-bezier(.25,.01,.25,1)';

    const { originX, originY } = this.ob,
      originX2 = evt.center.x - this.screenWidth / 2 - document.body.scrollLeft,
      originY2 = evt.center.y - this.screenHeight / 2 - document.body.scrollTop;

    this.ob.originX = originX2;
    this.ob.originY = originY2;
    this.ob.translateX =
      this.ob.translateX + (originX2 - originX) * this.ob.scaleX;
    this.ob.translateY =
      this.ob.translateY + (originY2 - originY) * this.ob.scaleY;

    this.ob.scaleX = this.ob.scaleY = this.initScale * evt.scale;
  }

  onRotate(evt) {
    if (!this.props.enableRotate || this.ob.getAttribute('rate') >= 3.5) {
      return false;
    }

    this.ob.style.webkitTransition = 'cubic-bezier(.25,.01,.25,1)';

    this.ob.rotateZ += evt.angle;
  }

  onLongTap() {
    this.props.longTap && this.props.longTap();
  }

  onMultipointEnd(evt) {
    // translate to normal
    this.changeIndex(this.state.current);

    if (!this.ob) {
      return;
    }

    this.ob.style.webkitTransition = '300ms ease';

    const { maxScale } = this.props,
      isLongPic = this.ob.getAttribute('long');
    // scale to normal
    if (this.ob.scaleX < 1) {
      this.restore(false);
    }
    if (this.ob.scaleX > maxScale && !isLongPic) {
      this.setScale(maxScale);
    }

    // rotate to normal
    let rotation = this.ob.rotateZ % 360,
      rate = this.ob.getAttribute('rate');

    if (rotation < 0) {
      rotation = 360 + rotation;
    }
    this.ob.rotateZ = rotation;

    if (rotation > 0 && rotation < 45) {
      this.ob.rotateZ = 0;
    } else if (rotation >= 315) {
      this.ob.rotateZ = 360;
    } else if (rotation >= 45 && rotation < 135) {
      this.ob.rotateZ = 90;
      this.setScale(rate);
    } else if (rotation >= 135 && rotation < 225) {
      this.ob.rotateZ = 180;
    } else if (rotation >= 225 && rotation < 315) {
      this.ob.rotateZ = 270;
      this.setScale(rate);
    }
  }

  onDoubleTap(evt) {
    if (this.props.disableDoubleTap) {
      return false;
    }

    const { origin } = evt,
      originX = origin[0] - this.screenWidth / 2 - document.body.scrollLeft,
      originY = origin[1] - this.screenHeight / 2 - document.body.scrollTop,
      isLongPic = this.ob.getAttribute('long');

    if (this.ob.scaleX === 1) {
      !isLongPic && (this.ob.translateX = this.ob.originX = originX);
      !isLongPic && (this.ob.translateY = this.ob.originY = originY);
      this.setScale(
        isLongPic ? this.screenWidth / this.ob.width : this.props.maxScale
      );
    } else {
      this.ob.translateX = this.ob.originX;
      this.ob.translateY = this.ob.originY;
      this.setScale(1);
    }

    // console.log('origin',this.ob.originX, this.ob.originY);
  }
  setStyle(styleData) {
    const elementStyle = findDOMNode(this.refs.page).style;
    for (const styleName in styleData) {
      elementStyle[styleName] = styleData[styleName];
    }
  }
  bindStyle(current) {
    this.setState({ current }, () => {
      this.ob && this.restore();
      this.ob = findDOMNode(`view${current}`);
      if (this.ob && !this.ob.scaleX) {
        Transform(this.ob);
      }
      // ease hide page number
      const page = this.refs.page;
      const style = {
        opacity: 0,
        transition: 'opacity 0.2s',
        webkitTransition: 'opacity 0.2s',
      };
      if (page) {
        this.setStyle({ opacity: 1 });
        setTimeout(() => {
          this.setStyle(style);
        }, 2000);
      }
    });
  }

  changeIndex(current, ease = true) {
    ease && (this.list.style.webkitTransition = '300ms ease');
    this.list.translateX = -current * (this.screenWidth + this.props.gap);

    this.props.onChange && this.props.onChange(current);
  }

  setScale(size) {
    this.ob.style.webkitTransition = '300ms ease-in-out';
    this.ob.scaleX = this.ob.scaleY = size;
  }

  restore(rotate = true) {
    this.ob.translateX = this.ob.translateY = 0;
    !!rotate && (this.ob.rotateZ = 0);
    this.ob.scaleX = this.ob.scaleY = 1;
    this.ob.originX = this.ob.originY = 0;
  }

  endAnimation() {
    this.list.style.webkitTransition = '0';
    this.ob && this.ob.style && (this.ob.style.webkitTransition = '0');
  }

  checkBoundary(deltaX = 0, deltaY = 0) {
    // console.log(this.ob.width, this.ob.height);
    const {
      scaleX,
      translateX,
      translateY,
      originX,
      originY,
      width,
      height,
    } = this.ob,
      rate = this.ob.getAttribute('rate');

    if (scaleX !== 1 || scaleX !== rate) {
      // include long picture
      const rangeLeft = (scaleX - 1) * (width / 2 + originX) + originX,
        rangeRight = -(scaleX - 1) * (width / 2 - originX) + originX,
        rangeUp = (scaleX - 1) * (height / 2 + originY) + originY,
        rangeDown = -(scaleX - 1) * (height / 2 - originY) + originY;

      // console.log(rangeLeft, rangeRight, rangeUp, rangeDown);

      if (
        translateX + deltaX <= rangeLeft &&
        translateX + deltaX >= rangeRight &&
        translateY + deltaY <= rangeUp &&
        translateY + deltaY >= rangeDown
      ) {
        return true;
      }
    }
    return false;
  }
}
ImageView.propTypes = {
  gap: PropTypes.number,
  maxScale: PropTypes.number,
  current: PropTypes.number,
  imageList: PropTypes.array.isRequired,
  disablePageNum: PropTypes.bool,
  disablePinch: PropTypes.bool,
  enableRotate: PropTypes.bool,
  disableDoubleTap: PropTypes.bool,
  longTap: PropTypes.func,
  close: PropTypes.func.isRequired,
  changeIndex: PropTypes.func,
  initCallback: PropTypes.func,

};
export default ImageView;
